<?php

class SimpleTreeWidget extends CInputWidget
{
    private $baseUrl;
                    
    public $ajaxUrl;
    public $id = 'simpletree_widget';
    public $model;
    public $modelPropertyName = 'title';
    public $modelPropertyId = 'id';
    public $modelPropertyParentId = 'id_parent';
    public $modelPropertyPosition = 'position';
    public $theme = 'default';
    public $onFocus;
    public $onCreate;
    public $onMove;
    public $onRemove;
    public $onRename;
    
    public function run()
    {
        //if this is an Ajax request, do Ajax and die.
        //It is recommended to change $ajaxUrl to call SimpleTreeWidget::performAjax() directly from a controller.
        if (Yii::app()->request->isAjaxRequest && isset($_POST['simpletree']))
        {       
            self::performAjax();
            die;
        }
        //register assets
        $clientScript = Yii::app()->getClientScript(); 
        $dir = dirname(__FILE__).'/SimpleTree';
        $this->baseUrl = Yii::app()->getAssetManager()->publish($dir);
        $clientScript->registerCoreScript('jquery');
        $clientScript->registerScriptFile($this->baseUrl . '/_lib/jquery.cookie.js', CClientScript::POS_HEAD);
        $clientScript->registerScriptFile($this->baseUrl . '/_lib/jquery.hotkeys.js', CClientScript::POS_HEAD);
        $clientScript->registerScriptFile($this->baseUrl . '/jquery.jstree.js', CClientScript::POS_HEAD);
        
        
        //create container node
        echo "<div id='$this->id'>node</div>";
        $clientScript->registerScript('createJtree','jQuery("#'.$this->id.'").jstree({
        "plugins" : [ "themes", "json_data", "ui", "crrm", "cookies", "dnd", "types", "hotkeys", "contextmenu" ],
            "json_data" : {
                "data" : [
                    '.$this->loadData().'
                ]
            },
            "themes" : {
                "theme" : "'.$this->theme.'",
                "url" : "'.$this->baseUrl.'/themes/'.$this->theme.'/style.css"
            }
        })
        .bind("create.jstree", function (e, data) {
            $.post(
                "'.$this->ajaxUrl.'", 
                { 
                    "simpletree" : 1,
                    "operation" : "create_node", 
                    "id" : data.rslt.parent.attr("id").replace("node_",""), 
                    "position" : data.rslt.position,
                    "title" : data.rslt.name,
                    "type" : data.rslt.obj.attr("rel"),
                    '.$this->getEnvironment().'
                }, 
                function (r) {
                    if(r.status) {
                        $(data.rslt.obj).attr("id", "node_" + r.id);
                        '.$this->onCreate.'
                    }
                    else {
                        console.log(r);
                        $.jstree.rollback(data.rlbk);
                    }
                }
            );
        })
        .bind("remove.jstree", function (e, data) {
            data.rslt.obj.each(function () {
                $.ajax({
                    async : false,
                    type: \'POST\',
                    url: "'.$this->ajaxUrl.'",
                    data : { 
                        "simpletree" : 1,
                        "operation" : "remove_node", 
                        "id" : this.id.replace("node_",""),
                        '.$this->getEnvironment().'
                    }, 
                    success : function (r) {
                        if(!r.status) {
                            data.inst.refresh();
                        }else{
                        
                            '.$this->onRemove.'
                        }
                    }
                });
            });
        })
        .bind("rename.jstree", function (e, data) {
            $.post(
                "'.$this->ajaxUrl.'", 
                { 
                    "simpletree" : 1,
                    "operation" : "rename_node", 
                    "id" : data.rslt.obj.attr("id").replace("node_",""),
                    "title" : data.rslt.new_name,
                    '.$this->getEnvironment().'
                }, 
                function (r) {
                    if(!r.status) {
                        $.jstree.rollback(data.rlbk);
                    }else{
                        '.$this->onRename.'
                    }
                }
            );
        })
        .bind("move_node.jstree", function (e, data) {
            data.rslt.o.each(function (i) {
                $.ajax({
                    async : false,
                    type: \'POST\',
                    url: "'.$this->ajaxUrl.'",
                    data : { 
                        "simpletree" : 1,
                        "operation" : "move_node", 
                        "id" : $(this).attr("id").replace("node_",""), 
                        "ref" : data.rslt.np.attr("id").replace("node_",""), 
                        "position" : data.rslt.cp + i,
                        "title" : data.rslt.name,
                        "copy" : data.rslt.cy ? 1 : 0,
                        '.$this->getEnvironment().'
                    },
                    success : function (r) {
                        console.log(r);
                        if(!r.status) {
                            $.jstree.rollback(data.rlbk);
                        }
                        else {
                            $(data.rslt.oc).attr("id", "node_" + r.id);
                            if(data.rslt.cy && $(data.rslt.oc).children("UL").length) {
                                data.inst.refresh(data.inst._get_parent(data.rslt.oc));
                            }
                            '.$this->onMove.'
                        }
                    }
                });
            });
        })
        .bind("set_focus.jstree", function (e, data){
            '.$this->onFocus.'
        });', CClientScript::POS_END);
        
    }

    public function getEnvironment()
    {
        return '
            "model" : "'.get_class($this->model).'",
            "modelPropertyId" : "'.$this->modelPropertyId.'",
            "modelPropertyParentId" : "'.$this->modelPropertyParentId.'",
            "modelPropertyPosition" : "'.$this->modelPropertyPosition.'",
            "modelPropertyName" : "'.$this->modelPropertyName.'",
        ';
    }
    
    public function loadData()
    {    
        $model = $this->model;
        //if string, replace with model
        if (is_string($model))
        {
            $model = $model::model();
            $this->model = $model;
        }
        
        if (!$model->{$this->modelPropertyId})
        {
            $model->{$this->modelPropertyId} = 0;
            $model->{$this->modelPropertyName} = 'root';
        }
        $hiearchy = ($this->createHierachy($model));
        return json_encode($hiearchy);        
    }

    private function createHierachy($model, $level = 0)
    {   
        $criteria=new CDbCriteria;
        $criteria->order=$this->modelPropertyPosition;
        $criteria->condition=$this->modelPropertyParentId.'='.$model->{$this->modelPropertyId};             

        $array['children'] = array();                                       
        $modelName = get_class($model);
        foreach ($modelName::model()->findAll($criteria) AS $Child)
            $array['children'][] = $this->createHierachy($Child, $level+1);
        $array['data'] = $model->{$this->modelPropertyName};
        $array['attr'] = array('id'=>'node_'.$model->{$this->modelPropertyId});
        if ($level == 0)
        {
            $array['state'] = 'open';
        }
        return $array;
    }
    
    static function performAjax()
    {
        $Model = new $_POST['model'];
        $method = '_'.$_POST['operation'];
        
        header("HTTP/1.0 200 OK");
        header('Content-type: text/json; charset=utf-8');
        header("Cache-Control: no-cache, must-revalidate");
        header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
        header("Pragma: no-cache");
        self::$method($_POST);
        
    }
    
    static function _create_node($params)
    {
        $Model = new $params['model'];
        $Model->$params['modelPropertyParentId'] = $params['id'];
        $Model->$params['modelPropertyName'] = $params['title'];
        @$Model->$params['modelPropertyPosition'] = $params['position'];
        
        if($Model->save())
            echo json_encode(array('status'=>1, 'id'=>$Model->$params['modelPropertyId']));
        else
            print_r($Model->getErrors());
    }
    
    static function _remove_node($params)
    {   
        self::_removeModelRecursively($params);
        echo json_encode(array('status'=>1));
    }
    
    static function _rename_node($params)
    {
        $Model = $params['model']::model()->findByPk($params['id']);
        $Model->$params['modelPropertyName'] = $params['title'];
        if($Model->save())
            echo json_encode(array('status'=>1));
    }
    
    
    static function _move_node($params)
    {   
        //get model
        $Model = $params['model']::model()->findByPk($params['id']);
        
        //copy and die?
        if($params['copy'])
        {
            $Model = self::_copy_node($Model, $params);
        }
        
        //get new siblings
        $criteria=new CDbCriteria;
        $criteria->order=$params['modelPropertyPosition'];
        $criteria->condition=$params['modelPropertyParentId'].'='.$params['ref'] . ' AND ' . $params['modelPropertyId'] . '!='.$params['id'];
        $siblings=$params['model']::model()->findAll($criteria);       
        
        //if item is moved to a higher position ID in it's current folder, make sure to substract it's old position as the item only exists once
        if ($Model->$params['modelPropertyParentId'] == $params['ref'] && $Model->$params['modelPropertyPosition'] < $params['position'])
            $params['position']--;
        
        //save model
        $Model->$params['modelPropertyPosition'] = $params['position'];
        $Model->$params['modelPropertyParentId'] = $params['ref'];
        $Model->save();
        
        //assign positions to siblings
        $i = 0;
        foreach ($siblings AS $Sibling)
        {
            //params position is reserved, so iterate by it
            if($i == $params['position'])
                $i++;
            $Sibling->$params['modelPropertyPosition'] = $i;
            $Sibling->save();
            $i++;
        }
        
        echo json_encode(array('status'=>1));
    }
    
    static function _copy_node($Model, $params, $inheritPosition = false)
    {
        $NewModel = new $params['model'];
        $NewModel->attributes = $Model->attributes;
        
        //copy these, even if they're unsafe values
        $NewModel->{$params['modelPropertyName']} = $Model->{$params['modelPropertyName']};
        $NewModel->{$params['modelPropertyParentId']} = $params['ref'];
        $NewModel->{$params['modelPropertyPosition']} = $inheritPosition? $Model->{$params['modelPropertyPosition']} : $params['position'];
        
        if ($NewModel->save())
        {
            //copy children
            foreach ($params['model']::model()->findAllByAttributes(array($params['modelPropertyParentId']=>$Model->{$params['modelPropertyId']})) AS $Child)
            {
                $params['ref'] = $NewModel->{$params['modelPropertyId']};
                self::_copy_node($Child, $params, true);
            }
            return $NewModel;
        }
    }
    
    static function _removeModelRecursively($params)
    {   
        $Model = $params['model']::model()->findByPk($params['id']);
        foreach($params['model']::model()->findAllByAttributes(array($params['modelPropertyParentId'] => $params['id'])) AS $Child)
        {
            $params['id'] = $Child->$params['modelPropertyId'];
            self::_removeModelRecursively($params);
        }
        $Model->delete();
    }
}


?>